
Imports System.Net.NetworkInformation
Imports System.Runtime.InteropServices
Imports System.Threading

Public Class WlanInterface
#Region "Fields"

    Private ReadOnly _client As WlanClient
    Private ReadOnly _eventQueue As New Queue(Of Object)()
    Private ReadOnly _eventQueueFilled As New AutoResetEvent(False)
    Private _info As Wlan.WlanInterfaceInfo

#End Region

#Region "Properties"

    Public Property Autoconf() As Boolean
        Get
            Return (GetInterfaceInt(Wlan.WlanIntfOpcode.AutoconfEnabled) <> 0)
        End Get
        Set(ByVal value As Boolean)
            SetInterfaceInt(Wlan.WlanIntfOpcode.AutoconfEnabled, If(Value, 1, 0))
        End Set
    End Property

    Public Property BssType() As Wlan.Dot11BssType
        Get
            Return CType(GetInterfaceInt(Wlan.WlanIntfOpcode.BssType), Wlan.Dot11BssType)
        End Get
        Set(ByVal value As Wlan.Dot11BssType)
            SetInterfaceInt(Wlan.WlanIntfOpcode.BssType, CInt(Value))
        End Set
    End Property

    Public ReadOnly Property Channel() As Integer
        Get
            Return GetInterfaceInt(Wlan.WlanIntfOpcode.ChannelNumber)
        End Get
    End Property

    Public ReadOnly Property CurrentConnection() As Wlan.WlanConnectionAttributes
        Get
            Dim num As Integer
            Dim ptr As IntPtr
            Dim type As Wlan.WlanOpcodeValueType
            Dim attributes As New Wlan.WlanConnectionAttributes()
            Dim code As Integer = Wlan.WlanQueryInterface(_client.ItsClientHandle, _info.interfaceGuid, Wlan.WlanIntfOpcode.CurrentConnection, IntPtr.Zero, num, ptr, type)
            ' 0x139F is the code returned when not connected
            If code <> &H139F Then
                Try
                    attributes = CType(Marshal.PtrToStructure(ptr, GetType(Wlan.WlanConnectionAttributes)), Wlan.WlanConnectionAttributes)
                Finally
                    Wlan.WlanFreeMemory(ptr)
                End Try
            End If
            Return attributes
        End Get
    End Property

    Public ReadOnly Property CurrentOperationMode() As Wlan.Dot11OperationMode
        Get
            Return CType(GetInterfaceInt(Wlan.WlanIntfOpcode.CurrentOperationMode), Wlan.Dot11OperationMode)
        End Get
    End Property

    Public ReadOnly Property InterfaceDescription() As String
        Get
            Return _info.interfaceDescription
        End Get
    End Property

    Public ReadOnly Property InterfaceGuid() As Guid
        Get
            Return _info.interfaceGuid
        End Get
    End Property

    'Public ReadOnly Property InterfaceName() As Guid
    '    Get
    '        Return NetworkInterface.Name
    '    End Get
    'End Property

    Public ReadOnly Property InterfaceState() As Wlan.WlanInterfaceState
        Get
            Return CType(GetInterfaceInt(Wlan.WlanIntfOpcode.InterfaceState), Wlan.WlanInterfaceState)
        End Get
    End Property

    Public ReadOnly Property NetworkInterface() As NetworkInterface
        Get
            Dim nd As NetworkInterface() = NetworkInterface.GetAllNetworkInterfaces()
            For Each interface2 As NetworkInterface In nd
                If interface2.NetworkInterfaceType <> NetworkInterfaceType.Loopback Then
                    Dim guid As New Guid(interface2.Id)
                    If guid.Equals(_info.interfaceGuid) Then
                        Return interface2
                    End If
                End If
            Next
            'We haven't found one yet, create one.

            'return null;
            Return New SudoInterface(Me)
        End Get
    End Property

    Public ReadOnly Property Rssi() As Integer
        Get
            Return GetInterfaceInt(Wlan.WlanIntfOpcode.Rssi)
        End Get
    End Property

#End Region

#Region "Event Related"

#Region "Event Fields"

    Private _queueEvents As Boolean

#End Region

#Region "Events"

    Public Event WlanConnectionNotification As WlanConnectionNotificationEventHandler

    Public Event WlanNotification As WlanNotificationEventHandler

    Public Event WlanReasonNotification As WlanReasonNotificationEventHandler

#End Region

#End Region

#Region "Delegates"

    Public Delegate Sub WlanConnectionNotificationEventHandler(ByVal notifyData As Wlan.WlanNotificationData, ByVal connNotifyData As Wlan.WlanConnectionNotificationData)

    Public Delegate Sub WlanNotificationEventHandler(ByVal notifyData As Wlan.WlanNotificationData)

    Public Delegate Sub WlanReasonNotificationEventHandler(ByVal notifyData As Wlan.WlanNotificationData, ByVal reasonCode As Wlan.WlanReasonCode)

#End Region

#Region "Constructors"

    Public Sub New(ByVal client As WlanClient, ByVal info As Wlan.WlanInterfaceInfo)
        _client = client
        _info = info
    End Sub

#End Region

#Region "Public Methods"

    Public Sub Connect(ByVal connectionMode As Wlan.WlanConnectionMode, ByVal bssType As Wlan.Dot11BssType, ByVal ssid As Wlan.Dot11Ssid, ByVal flags As Wlan.WlanConnectionFlags)
        Dim parameters2 As New Wlan.WlanConnectionParameters() With { _
          .wlanConnectionMode = connectionMode, _
          .dot11SsidPtr = Marshal.AllocHGlobal(Marshal.SizeOf(ssid)), _
          .dot11BssType = bssType, _
          .flags = flags _
        }
        Dim connectionParams As Wlan.WlanConnectionParameters = parameters2
        Me.Connect(connectionParams)
        Marshal.StructureToPtr(ssid, connectionParams.dot11SsidPtr, False)
        Marshal.DestroyStructure(connectionParams.dot11SsidPtr, ssid.[GetType]())
        Marshal.FreeHGlobal(connectionParams.dot11SsidPtr)
    End Sub

    Public Function ConnectSynchronously(ByVal connectionMode As Wlan.WlanConnectionMode, ByVal bssType As Wlan.Dot11BssType, ByVal profile As String, ByVal connectTimeout As Integer) As Boolean
        _queueEvents = True
        Try
            Connect(connectionMode, bssType, profile)
            While _queueEvents AndAlso _eventQueueFilled.WaitOne(connectTimeout, True)
                SyncLock _eventQueue
                    While _eventQueue.Count <> 0
                        Dim obj2 As Object = _eventQueue.Dequeue()
                        If TypeOf obj2 Is WlanConnectionNotificationEventData Then
                            Dim data As WlanConnectionNotificationEventData = CType(obj2, WlanConnectionNotificationEventData)
                            If ((data.NotifyData.notificationSource <> Wlan.WlanNotificationSource.Acm) OrElse (CInt(data.NotifyData.NotificationCode) <> 10)) OrElse data.ConnNotifyData.profileName <> profile Then
                                Exit While
                            End If
                            Return True
                        End If
                    End While
                    Continue While
                End SyncLock
            End While
        Finally
            _queueEvents = False
            _eventQueue.Clear()
        End Try
        Return False
    End Function

    Public Sub DeleteProfile(ByVal profileName As String)
        Wlan.WlanDeleteProfile(_client.ItsClientHandle, _info.interfaceGuid, profileName, IntPtr.Zero)
    End Sub

    Public Function GetAvailableNetworkList(ByVal flags As Wlan.WlanGetAvailableNetworkFlags) As IEnumerable(Of Wlan.WlanAvailableNetwork)
        Dim ptr As IntPtr
        Dim networkArray As Wlan.WlanAvailableNetwork()
        Try
            Wlan.WlanGetAvailableNetworkList(_client.ItsClientHandle, _info.interfaceGuid, flags, IntPtr.Zero, ptr)
            networkArray = ConvertAvailableNetworkListPtr(ptr)
        Finally
            Wlan.WlanFreeMemory(ptr)
        End Try
        Return networkArray
    End Function

    Public Function GetNetworkBssList() As IEnumerable(Of Wlan.WlanBssEntryN)
        Dim ptr As IntPtr
        Dim entryArray As Wlan.WlanBssEntryN()
        Try
            Wlan.WlanGetNetworkBssList(_client.ItsClientHandle, _info.interfaceGuid, IntPtr.Zero, Wlan.Dot11BssType.Any, False, IntPtr.Zero, ptr)
            entryArray = ConvertBssListPtr(ptr)
        Finally
            Wlan.WlanFreeMemory(ptr)
        End Try
        Return entryArray
    End Function

    Public Function GetNetworkBssList(ByVal ssid As Wlan.Dot11Ssid, ByVal bssType As Wlan.Dot11BssType, ByVal securityEnabled As Boolean) As Wlan.WlanBssEntryN()
        Dim entryArray As Wlan.WlanBssEntryN()
        Dim ptr As IntPtr = Marshal.AllocHGlobal(Marshal.SizeOf(ssid))
        Marshal.StructureToPtr(ssid, ptr, False)
        Try
            Dim ptr2 As IntPtr
            Try
                Wlan.WlanGetNetworkBssList(_client.ItsClientHandle, _info.interfaceGuid, ptr, bssType, securityEnabled, IntPtr.Zero, ptr2)
                entryArray = ConvertBssListPtr(ptr2)
            Finally
                Wlan.WlanFreeMemory(ptr2)
            End Try
        Finally
            Marshal.FreeHGlobal(ptr)
        End Try
        Return entryArray
    End Function

    Public Function GetProfiles() As Wlan.WlanProfileInfo()
        Dim ptr As IntPtr
        Dim infoArray2 As Wlan.WlanProfileInfo()
        Try
            Wlan.WlanGetProfileList(_client.ItsClientHandle, _info.interfaceGuid, IntPtr.Zero, ptr)
            Dim [structure] As Wlan.WlanProfileInfoListHeader = CType(Marshal.PtrToStructure(ptr, GetType(Wlan.WlanProfileInfoListHeader)), Wlan.WlanProfileInfoListHeader)
            Dim infoArray As Wlan.WlanProfileInfo() = New Wlan.WlanProfileInfo(CInt([structure].numberOfItems - 1)) {}
            Dim num As Long = ptr.ToInt64() + Marshal.SizeOf([structure])
            For i As Integer = 0 To CInt([structure].numberOfItems - 1)
                Dim info As Wlan.WlanProfileInfo = CType(Marshal.PtrToStructure(New IntPtr(num), GetType(Wlan.WlanProfileInfo)), Wlan.WlanProfileInfo)
                infoArray(i) = info
                num += Marshal.SizeOf(info)
            Next
            infoArray2 = infoArray
        Finally
            Wlan.WlanFreeMemory(ptr)
        End Try
        Return infoArray2
    End Function

    Public Function GetProfileXml(ByVal profileName As String) As String
        Dim ptr As IntPtr
        Dim flags As Wlan.WlanProfileFlags
        Dim access As Wlan.WlanAccess
        Dim str As String
        Try
            Wlan.WlanGetProfile(_client.ItsClientHandle, _info.interfaceGuid, profileName, IntPtr.Zero, ptr, flags, access)
            str = Marshal.PtrToStringUni(ptr)
        Finally
            Wlan.WlanFreeMemory(ptr)
        End Try
        Return str
    End Function

    Public Sub Scan()
       Wlan.WlanScan(_client.ItsClientHandle, _info.interfaceGuid, IntPtr.Zero, IntPtr.Zero, IntPtr.Zero)
    End Sub

    Public Function SetProfile(ByVal flags As Wlan.WlanProfileFlags, ByVal profileXml As String, ByVal overwrite As Boolean) As Wlan.WlanReasonCode
        Dim code As Wlan.WlanReasonCode
        Wlan.WlanSetProfile(_client.ItsClientHandle, _info.interfaceGuid, flags, profileXml, Nothing, overwrite, IntPtr.Zero, code)
        Return code
    End Function

#End Region

#Region "Private Methods"

    Private Sub Connect(ByVal connectionParams As Wlan.WlanConnectionParameters)
        Wlan.WlanConnect(_client.ItsClientHandle, _info.interfaceGuid, connectionParams, IntPtr.Zero)
    End Sub

    Private Sub Connect(ByVal connectionMode As Wlan.WlanConnectionMode, ByVal bssType As Wlan.Dot11BssType, ByVal profile As String)
        Dim parameters2 As New Wlan.WlanConnectionParameters() With { _
          .wlanConnectionMode = connectionMode, _
          .profile = profile, _
          .dot11BssType = bssType, _
          .flags = 0 _
        }
        Dim connectionParams As Wlan.WlanConnectionParameters = parameters2
        Connect(connectionParams)
    End Sub

    Private Function ConvertAvailableNetworkListPtr(ByVal availNetListPtr As IntPtr) As Wlan.WlanAvailableNetwork()
        Dim header As Wlan.WlanAvailableNetworkListHeader = CType(Marshal.PtrToStructure(availNetListPtr, GetType(Wlan.WlanAvailableNetworkListHeader)), Wlan.WlanAvailableNetworkListHeader)
        Dim num As Long = availNetListPtr.ToInt64() + Marshal.SizeOf(GetType(Wlan.WlanAvailableNetworkListHeader))
        Dim networkArray As Wlan.WlanAvailableNetwork() = New Wlan.WlanAvailableNetwork(CInt(header.numberOfItems - 1)) {}
        For i As Integer = 0 To CInt(header.numberOfItems - 1)
            networkArray(i) = CType(Marshal.PtrToStructure(New IntPtr(num), GetType(Wlan.WlanAvailableNetwork)), Wlan.WlanAvailableNetwork)
            num += Marshal.SizeOf(GetType(Wlan.WlanAvailableNetwork))
        Next
        Return networkArray
    End Function

    Private Function ConvertBssListPtr(ByVal bssListPtr As IntPtr) As Wlan.WlanBssEntryN()
        Dim header As Wlan.WlanBssListHeader = CType(Marshal.PtrToStructure(bssListPtr, GetType(Wlan.WlanBssListHeader)), Wlan.WlanBssListHeader)
        Dim num As Long = bssListPtr.ToInt64() + Marshal.SizeOf(GetType(Wlan.WlanBssListHeader))
        Dim entryArray As Wlan.WlanBssEntryN() = New Wlan.WlanBssEntryN(CInt(header.numberOfItems - 1)) {}
        For i As Integer = 0 To CInt(header.numberOfItems - 1)
            entryArray(i) = New Wlan.WlanBssEntryN(CType(Marshal.PtrToStructure(New IntPtr(num), GetType(Wlan.WlanBssEntry)), Wlan.WlanBssEntry))

            Dim size As Integer = CInt(entryArray(i).BaseEntry.ieSize)
            entryArray(i).IEs = New Byte(size - 1) {}

            Marshal.Copy(New IntPtr(num + entryArray(i).BaseEntry.ieOffset), entryArray(i).IEs, 0, size)

            num += Marshal.SizeOf(GetType(Wlan.WlanBssEntry))
        Next
        Return entryArray
    End Function

    Private Sub EnqueueEvent(ByVal queuedEvent As Object)
        SyncLock _eventQueue
            _eventQueue.Enqueue(queuedEvent)
        End SyncLock
        _eventQueueFilled.[Set]()
    End Sub

    Private Function GetInterfaceInt(ByVal opCode As Wlan.WlanIntfOpcode) As Integer
        Dim ptr As IntPtr
        Dim num As Integer
        Dim type As Wlan.WlanOpcodeValueType
        Dim num2 As Integer
        Try
            Wlan.WlanQueryInterface(_client.ItsClientHandle, _info.interfaceGuid, opCode, IntPtr.Zero, num, ptr, type)
            num2 = Marshal.ReadInt32(ptr)
        Finally
            Wlan.WlanFreeMemory(ptr)
        End Try
        Return num2
    End Function

    Friend Sub OnWlanConnection(ByVal notifyData As Wlan.WlanNotificationData, ByVal connNotifyData As Wlan.WlanConnectionNotificationData)
        RaiseEvent WlanConnectionNotification(notifyData, connNotifyData)
        If _queueEvents Then
            Dim data2 As New WlanConnectionNotificationEventData() With _
            { _
             .NotifyData = notifyData, _
             .ConnNotifyData = connNotifyData _
            }
            Dim queuedEvent As WlanConnectionNotificationEventData = data2
            EnqueueEvent(queuedEvent)
        End If
    End Sub

    Friend Sub OnWlanNotification(ByVal notifyData As Wlan.WlanNotificationData)
        RaiseEvent WlanNotification(notifyData)
    End Sub

    Friend Sub OnWlanReason(ByVal notifyData As Wlan.WlanNotificationData, ByVal reasonCode As Wlan.WlanReasonCode)
        RaiseEvent WlanReasonNotification(notifyData, reasonCode)
        If _queueEvents Then
            Dim data2 As New WlanReasonNotificationData() With _
            { _
              .NotifyData = notifyData, _
              .ReasonCode = reasonCode _
            }
            Dim queuedEvent As WlanReasonNotificationData = data2
            EnqueueEvent(queuedEvent)
        End If
    End Sub

    Private Sub SetInterfaceInt(ByVal opCode As Wlan.WlanIntfOpcode, ByVal value As Integer)
        Dim ptr As IntPtr = Marshal.AllocHGlobal(4)
        Marshal.WriteInt32(ptr, value)
        Try
            Wlan.WlanSetInterface(_client.ItsClientHandle, _info.interfaceGuid, opCode, 4, ptr, IntPtr.Zero)
        Finally
            Marshal.FreeHGlobal(ptr)
        End Try
    End Sub

    <StructLayout(LayoutKind.Sequential)> _
    Private Structure WlanConnectionNotificationEventData
        Public NotifyData As Wlan.WlanNotificationData
        Public ConnNotifyData As Wlan.WlanConnectionNotificationData
    End Structure

    <StructLayout(LayoutKind.Sequential)> _
    Private Structure WlanReasonNotificationData
        Public NotifyData As Wlan.WlanNotificationData
        Public ReasonCode As Wlan.WlanReasonCode
    End Structure

#End Region
End Class

